/*
    ---------------------------------------------------------------------- copyright and license ---

    file: src/interceptor.cpp

    Copyright © 2014, 2016—2017, 2019, 2020 Van de Bugger.

    This file is a part of Tapper.

    Tapper is free software: you can redistribute it and/or modify it under the terms of the GNU
    General Public License as published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Tapper is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
    even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License along with Tapper.  If not,
    see <http://www.gnu.org/licenses/>.

    SPDX-License-Identifier: GPL-3.0-or-later

    ---------------------------------------------------------------------- copyright and license ---
*/

/**
    @file
    `interceptor_t` abstract class implementation.
**/

#include "interceptor.hpp"

namespace tapper {

interceptor_t::key_state_t
interceptor_t::key_state(
    bool pressed
) {
    return pressed ? key_state_t::pressed : key_state_t::released;
};

/**
    Converts key code to key name.

    The method is virtual, but it is assumed the descendants will not override it, they should
    override `_key_name` instead. Default implementation seems returns string composed of key code
    and key name (returned by `_key_name`) joined with colon if key name is not empy.
**/
string_t
interceptor_t::key_name(
    key_t key
) {
    auto name = _key_name( key );
    return std::to_string( key ) + ( name.empty() ? "" : ":" + name );
};

void
interceptor_t::set_on_event(
    on_event_t on_event
) {
    _on_event = on_event;
};

/**
    Returns key name for specified key code.

    This is default implementation, it always returns empty string. It is assumed the descendants
    override it.
**/
string_t
interceptor_t::_key_name(
    key_t key
) {
    return "";
};

}; // namespace tapper

// end of file //
