/*
    ---------------------------------------------------------------------- copyright and license ---

    file: src/layouter-x.cpp

    Copyright © 2014, 2016—2017, 2019, 2020 Van de Bugger.

    This file is a part of Tapper.

    Tapper is free software: you can redistribute it and/or modify it under the terms of the GNU
    General Public License as published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Tapper is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
    even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License along with Tapper.  If not,
    see <http://www.gnu.org/licenses/>.

    SPDX-License-Identifier: GPL-3.0-or-later

    ---------------------------------------------------------------------- copyright and license ---
*/

/**
    @file
    `layouter::x_t` class implementation.
**/

#include "layouter-x.hpp"

#include <cassert>

namespace tapper {
namespace layouter {

// -------------------------------------------------------------------------------------------------
// x_t
// -------------------------------------------------------------------------------------------------

x_t::x_t(
) :
    OBJECT_T(),
    _kb( _display )
{
}; // ctor

string_t
x_t::type(
) {
    return "Xkb";
}; //type

uint_t
x_t::layout_max(
) {
    return x::layout_max;
};

time_t
x_t::get_autorepeat_delay(
) {
    auto rate = _kb.autorepeat_rate();
    return rate.timeout;
};

strings_t
x_t::layouts(
) {
    x::kb_t::desc_p kb_desc( _kb.desc() );
    return kb_desc->group_names();
}; // layouts

void
x_t::activate(
    uint_t layout
) {
    _kb.lock_group( layout );
    if ( _bell ) {
        _kb.bell();
    }; // if
    _kb.display().flush();
}; // activate

}; // namespace layouter
}; // namespace tapper

// end of file //
