/*
    ---------------------------------------------------------------------- copyright and license ---

    file: src/li.hpp

    Copyright © 2014, 2016—2017, 2019, 2020 Van de Bugger.

    This file is a part of Tapper.

    Tapper is free software: you can redistribute it and/or modify it under the terms of the GNU
    General Public License as published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Tapper is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
    even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License along with Tapper.  If not,
    see <http://www.gnu.org/licenses/>.

    SPDX-License-Identifier: GPL-3.0-or-later

    ---------------------------------------------------------------------- copyright and license ---
*/

/**
    @file
    libinput wrappers interface.
**/

#ifndef _LI_HPP_
#define _LI_HPP_

#include "base.hpp"
#include "posix.hpp"

#include <functional>
#include <map>

#include <libinput.h>

struct udev;

namespace tapper {
/// libinput wrappers.
namespace li {

    // ---------------------------------------------------------------------------------------------
    // error_t
    // ---------------------------------------------------------------------------------------------

    class error_t : public std::runtime_error {
        public :
            using std::runtime_error::runtime_error;
    }; // class error_t

    // ---------------------------------------------------------------------------------------------
    // udev_t
    // ---------------------------------------------------------------------------------------------

    class udev_t {

        public :

            typedef udev_t myself_t;

            udev_t();
            udev_t( myself_t const & that );
            ~udev_t();
            myself_t & operator =( myself_t const & that );

        public :

            udev * rep { nullptr };

    }; // class udev_t

    class context_t : public object_t {

        public :

            class event_t;

            class event_t : public object_t {
                friend class context_t;
                public :
                    typedef enum libinput_event_type type_t;
                    type_t type() const;
                private :
                    explicit event_t( context_t & context );
                    ~event_t();
                private :
                    libinput_event * rep;
            }; // class event_t

            typedef std::function< void( libinput_event * ) > on_event_t;

        public :

            context_t(
                on_event_t          on_event,
                string_t const &    seat,
                udev_t const &      udev = udev_t()
            );
            virtual ~context_t();
            void enable();
            void disable();
            int fd();
            virtual int  open( string_t const & path, int flags );
            virtual void close( int fd );

        public :

            libinput *  rep { nullptr };

        private :

            enum state_t {
                inited,
                enabled,
                disabled
            }; // enum state_t

            class thread_t : public posix::thread_t {
                friend class context_t;
                private :
                    explicit thread_t( context_t & context );
                    virtual void body() override;
                private :
                    context_t & _context;
            };

            typedef std::map< int, string_t > files_t;

        private :

            on_event_t  _on_event;
            state_t     _state { inited };
            string_t    _seat;
            thread_t    _thread;
            files_t     _files;

    }; // class context_t

    std::ostream &
    operator <<(
        std::ostream &              stream,
        context_t::event_t::type_t  type
    );

    std::ostream &
    operator <<(
        std::ostream &              stream,
        libinput_log_priority       priority
    );

}; // namespace li
}; // namespace tapper

#endif // _LI_HPP_

// end of file //
