/*
    ---------------------------------------------------------------------- copyright and license ---

    file: src/main.cpp

    Copyright © 2014, 2016—2017, 2019, 2020 Van de Bugger.

    This file is a part of Tapper.

    Tapper is free software: you can redistribute it and/or modify it under the terms of the GNU
    General Public License as published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Tapper is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
    even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License along with Tapper.  If not,
    see <http://www.gnu.org/licenses/>.

    SPDX-License-Identifier: GPL-3.0-or-later

    ---------------------------------------------------------------------- copyright and license ---
*/

/**
    @file
    `main` function is defined here.
**/

#include "config.h"

#include <exception>
#include <locale>

#include "base.hpp"
#include "app.hpp"

#if ENABLE_GNOME
    #include <glibmm/init.h>
    #include <giomm/init.h>
#endif // ENABLE_GNOME

using namespace tapper;

int
main(
    int     argc,
    char *  argv[]
) {
    int status = tapper::app_t::ok;
    try {
        /*
            Important, especially for debug build with Gnome support, because I do use Unicode
            characters in debug messages. In case of non-Unicode locale Glib's `operator <<`
            may throw an exception.
        */
        void( std::locale::global( std::locale( "" ) ) );
        #if ENABLE_GNOME
            /*
                Lazy initialization is too complicated, because it is hot clear if these functions
                are thread-safe, so let's initialize glibmm library here.
            */
            Glib::init();
            Gio::init();
        #endif // ENABLE_GNOME
        app_t app( argc, argv );
    } catch ( std::exception const & ex ) {
        eprint( priority_t::error, ex.what() );
            // Can't use `ERR` macro here: it will throw an exception,
            // while I just want to print the error message.
        status = tapper::app_t::runtime_error;
    } catch ( ... ) {
        eprint( priority_t::error, "Unknown exception occurred" );
        status = tapper::app_t::runtime_error;
    }; // try
    return status;
}; // main

// end of file //
