/*
    ---------------------------------------------------------------------- copyright and license ---

    file: src/settings.hpp

    Copyright © 2014, 2016—2017, 2019, 2020 Van de Bugger.

    This file is a part of Tapper.

    Tapper is free software: you can redistribute it and/or modify it under the terms of the GNU
    General Public License as published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Tapper is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
    even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License along with Tapper.  If not,
    see <http://www.gnu.org/licenses/>.

    SPDX-License-Identifier: GPL-3.0-or-later

    ---------------------------------------------------------------------- copyright and license ---
*/

/**
    @file
    `settings_t` class interface.
**/

#ifndef _SETTINGS_HPP_
#define _SETTINGS_HPP_

#include "base.hpp"

#include <map>

namespace tapper {

/**
    Tapper settings.

    Settings can be saved in GSettings (dconf) database and loaded back.
**/
struct settings_t {

    /** Bell status: disabled or enabled. **/
    enum class bell_t {
        unset = -1,
        disabled,
        enabled
    };

    /**
        Type of interceptor.
        The values must be aligned with the values in .gschema.xml file!
    **/
    enum class interceptor_t {
        unset = -1,
        Auto,           // `auto` is a C++ keyword.
        libinput,
        xrecord,
    };

    /**
        Type of layouter.
        The values must be aligned with the values in .gschema.xml file!
    **/
    enum class layouter_t {
        unset = -1,
        Auto,           // `auto` is a C++ keyword.
        gnome_shell,
        xkb,
    };

    /**
        Key/layout assignments.
    **/
    typedef std::map< key_t, uint_t >   assignments_t;

    static string_t   to_string( bell_t bell );
    static string_t   to_string( interceptor_t interceptor );
    static string_t   to_string( layouter_t layouter );

    static interceptor_t to_interceptor( string_t const & str );
    static layouter_t    to_layouter( string_t const & str );

    static strings_t  interceptors();
    static strings_t  layouters();

    static settings_t load();
    void              save() const;

    // Data fields:
    interceptor_t   interceptor { interceptor_t::unset };
    layouter_t      layouter { layouter_t::unset };
    bell_t          bell { bell_t::unset };
    assignments_t   assignments;

}; // struct settings_t

settings_t::assignments_t &
operator +=( settings_t::assignments_t & lhs, settings_t::assignments_t const & rhs );

settings_t &    operator +=( settings_t & lhs, settings_t const & rhs );
settings_t      operator +( settings_t const & lhs, settings_t const & rhs );

}; // namespace tapper

#endif // _SETTINGS_HPP_

// end of file //
