/*
    ---------------------------------------------------------------------- copyright and license ---

    file: src/types.cpp

    Copyright © 2014, 2016—2017, 2019, 2020 Van de Bugger.

    This file is a part of Tapper.

    Tapper is free software: you can redistribute it and/or modify it under the terms of the GNU
    General Public License as published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Tapper is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
    even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License along with Tapper.  If not,
    see <http://www.gnu.org/licenses/>.

    SPDX-License-Identifier: GPL-3.0-or-later

    ---------------------------------------------------------------------- copyright and license ---
*/

#include "types.hpp"

#include <algorithm>    // std::min, std::max

#include <cassert>

namespace tapper {

// -------------------------------------------------------------------------------------------------
// key_range_t
// -------------------------------------------------------------------------------------------------

key_range_t::key_range_t(
    uint_t _min,
    uint_t _max
) :
    min( _min ),
    max( _max )
{
    assert( _min <= _max );
};

/** Extend this range to include specified `key`. **/
key_range_t &
key_range_t::operator +=(
    key_t key
) {
    min = std::min( min, key );
    max = std::max( max, key );
    return * this;
};

/** Extend this range to include range `that`. **/
key_range_t &
key_range_t::operator +=(
    key_range_t const & that
) {
    min = std::min( min, that.min );
    max = std::max( max, that.max );
    return * this;
};

bool
key_range_t::includes(
    uint_t code
) const {
    return ( min <= code ) and ( code <= max );
};

std::ostream &
operator <<(
    std::ostream & stream,
    key_range_t    range
) {
    stream << "[" << range.min << ".." << range.max << "]";
    return stream;
};

}; // namespace tapper

// end of file //
